%"Copyright 2015 Isaac Jose Manjarres"

%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.

%The Mako Eye Tracking Toolbox is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%Name: analyzeData
%Parameters: beGazeData, rawData
%Description: The purpose of this function is to take a condensed BeGazeData 
%file in the form of a CSV and compare it to a rawData file. The means by which 
%this data is collected is described in the Eye-Tracker Validity Documentation
%PDF. The comparison is done to ensure that the data that we have collected is
%the same as the data collected from the BeGaze file. The comparison is done 
%by ensuring that the data samples match in terms of timestamps, x and y 
%positions of the left eye gaze, as well as the left eye pupil diameter.
function analyzeData(beGazeData, rawData)
  %First, we ensure that the data files exist
  if(exist(rawData) ~= 2 || exist(beGazeData) ~= 2)
    fprintf(['ERROR: One or both of the files that you have specified does',...
             ' not exist. Please ensure that the paths that you have',...
             ' specified are correct. The program will now terminate.\n']);
  end
  typeColumnNumber = 2; %column number of the type of data contained in the line
  %If it is of type MSG, it is just a message. If it is of type SMP, then it is
  %a data sample point to make sure that we have. 
  beGazeTimeColumn = 1;
  beGazeXColumn = 5;
  beGazeYColumn = 6;
  beGazeDiameterColumn = 4;
  rawDataTimeColumn = 1;
  rawDataXColumn = 2;
  rawDataYColumn = 3;
  rawDataDiameterColumn = 4;
  %We begin by opening both files once we know that they exist
  beGazeDataFD = fopen(beGazeData, 'r');
  rawDataFD = fopen(rawData, 'r');
  %Now, in both files it should be noted that the first line is a header, so 
  %this line is useless for data analysis. As a result, we can discard it. 
  currentLineBeGaze = fgets(beGazeDataFD);
  currentLineRawData = fgets(rawDataFD);
  %Now that we have obtained the first line, we can ignore them, so we move to 
  %the next line in both files
  currentLineBeGaze = fgets(beGazeDataFD);
  currentLineRawData = fgets(rawDataFD);
  beGazeLineNumber = 2;
  rawDataLineNumber = 2;
  matchedDataLines = 0;
  mismatchedLines = 0;
  %Now we begin the data analysis process since we have lines that contain data.
  %CAUTION: Since the eye tracker's iV_StartRecording function is called before 
  %iV_GetSample, it will include a first line that has a message that lets the 
  %user know at what time the eye tracker was started.
  while(ischar(currentLineRawData) && ischar(currentLineBeGaze))
    dataPointsBeGaze = strsplit(currentLineBeGaze, ',');
    %We check what type of data we're analyzing. If we are looking at a 
    %MSG type of data, then we can ignore it, since it's not a sample point
    dataType = dataPointsBeGaze(typeColumnNumber);
    %If we have a sample point, then we analyze it
    if(strcmpi('SMP', dataType))
      dataPointsRawData = strsplit(currentLineRawData, ',');
      validTimeStamps = isequal(str2double(...
                                dataPointsBeGaze(beGazeTimeColumn)),...
                        str2double(dataPointsRawData(rawDataTimeColumn)));
      validXCoord = isequal(str2double(dataPointsBeGaze(beGazeXColumn)) , ...
                    str2double(dataPointsRawData(rawDataXColumn)));
      validYCoord = isequal(str2double(dataPointsBeGaze(beGazeYColumn)) , ...
                    str2double(dataPointsRawData(rawDataYColumn)));
      %The iV_GetSample function only collects the diameter of the pupil with up
      %to two decimal places, but the BeGaze datalog uses, 3 decimal places, so
      %we use a factor of 0.01 to determine an upper and lower bound for the 
      %diameter, since the beGazeDiameter may have gotten rounded up or down
      beGazeEyeDiameter = round(str2double(dataPointsBeGaze(...
                                beGazeDiameterColumn)), 2);
      roundFactor = 0.01;
      upperBound = beGazeEyeDiameter + roundFactor;
      lowerBound = beGazeEyeDiameter - roundFactor;
      rawDataDiameter = str2double(dataPointsRawData(rawDataDiameterColumn));
      if(lowerBound <= rawDataDiameter && rawDataDiameter <= upperBound)
        validDiameter = 1;
      else
        validDiameter = 0;
      end
      %If all of these are valid, then we have a match.
      if(validTimeStamps && validXCoord && validYCoord && validDiameter)
        matchedDataLines = matchedDataLines + 1;
      %report mismatch to user
      else
        fprintf('Mismatch in BeGaze Line: %d and Raw Data Line: %d\n',... 
                beGazeLineNumber, rawDataLineNumber);
        mismatchedLines = mismatchedLines + 1;
      end
      currentLineBeGaze = fgets(beGazeDataFD);
      currentLineRawData = fgets(rawDataFD);
      beGazeLineNumber = beGazeLineNumber + 1;
      rawDataLineNumber = rawDataLineNumber + 1;
    %We are looking at a message line, so we only advance the BeGaze data
    %line count
    else
      beGazeLineNumber = beGazeLineNumber + 1;
      currentLineBeGaze = fgets(beGazeDataFD);
    end
  end
  %Report data back to user:
  fprintf(['Analysis complete.\nSummary:\nMatched Lines: %d\n',...
           'Mismatched Lines: %d\n'], matchedDataLines, mismatchedLines);
  return



