%"Copyright 2015 Isaac Jose Manjarres"

%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.

%The Mako Eye Tracking Toolbox is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%Name: testEyeTrackerValidity
%Parameters: pathToFixationCross
%The purpose of this function is to test the validity of the eye tracking device
%that we use. For our gaze data collection, we used the iView Red-m eye tracker. 
%The parameter, pathToFixationCross, is the path to an image that we used to 
%conduct the testing of the eye tracker's validity. More specifically, the 
%image is of a cross centered at the center of our 1680x1050 screen. This image
%is displayed, and gaze data is collected via the two data collection methods 
%described in the documentation file. It is necessary that an Infrastructure 
%directory exist, as well as all of its subdirectories. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%NOTE: If you wish to use this function, you must modify the following         %
%parameters:                                                                   %
%stimuliPath This is the path to the stimuli directory in the Infrastructure   %
%directory                                                                     %
%fileName This will be the name of the IDF that is created after the test      %
%physHeight This is the height of the display in centimeters                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function testEyeTrackerValidity(pathToFixationCross)
  %INITIALIZATION OF EYE TRACKER. DO NOT CHANGE THESE PARAMETERS!!!
  % --------------------------------------------------------------------------
  % ---- initialization
  % --------------------------------------------------------------------------
  warning('off', 'all');
  connected = 0;
  calibrated = 0; %Used later inside of calibration loop
  % add path variable to find the right header file and libraries
  path(path,'..\..\..\..\include')
  path(path,'..\..\..\..\bin');
  %defining the architecture of the libraries which needs to be used
  if computer('arch') == 'win32'
    includename = 'iViewXAPI.h';
    dllname = 'iViewXAPI.dll';
    libraryname = 'iViewXAPI';
  else 
    includename = 'iViewXAPI.h';
    dllname = 'iViewXAPI64.dll';
    libraryname = 'iViewXAPI64';
  end
  % --------------------------------------------------------------------------
  % ---- load the iView X API library
  % --------------------------------------------------------------------------
  loadlibrary(dllname, includename);
  %This initializes all of the necessary calibration information when starting 
  %up the device. These are parameters defined by the iViewX SDK and should 
  %NOT be changed
  % --------------------------------------------------------------------------
  % ---- initialize iView X API 
  % --------------------------------------------------------------------------
  % see the User Manual for detailed information due to setting up calibration
  [pSystemInfoData, pSampleData, pEventData, pAccuracyData, CalibrationData] = InitiViewXAPI();
  CalibrationData.method = int32(5);
  CalibrationData.visualization = int32(1);
  CalibrationData.displayDevice = int32(1); %We will use 1 since we are using
  %an external monitor to display our stimulus.
  CalibrationData.speed = int32(0);
  CalibrationData.autoAccept = int32(1);
  CalibrationData.foregroundBrightness = int32(250);
  CalibrationData.backgroundBrightness = int32(230);
  CalibrationData.targetShape = int32(2);
  CalibrationData.targetSize = int32(20);
  CalibrationData.targetFilename = int8('');
  pCalibrationData = libpointer('CalibrationStruct', CalibrationData);
  % ------------------------------------------------------------------------- 
  % ---- connect to iView X (eyetracking-server) 
  % -------------------------------------------------------------------------
  disp('Define Logger')
  calllib(libraryname, 'iV_SetLogger', int32(1), 'iViewXSDK_Matlab_Slideshow_Demo.txt')
  disp('Connect to iViewX (eyetracking-server)')
  ret = calllib(libraryname, 'iV_Connect', '127.0.0.1', int32(4444), '127.0.0.1', int32(5555));
  switch ret
  case 1
    connected = 1;
  case 104
    msgbox('Could not establish connection. Check if Eye Tracker is running', 'Connection Error', 'modal');
  case 105
    msgbox('Could not establish connection. Check the communication Ports', 'Connection Error', 'modal');
  case 123
    msgbox('Could not establish connection. Another Process is blocking the communication Ports', 'Connection Error', 'modal');
  case 200 
    msgbox('Could not establish connection.', 'Check if the Eye Tracker is installed and running', 'Connection Error', 'modal');
  otherwise
    msgbox('Could not establish connection', 'Connection Error', 'modal');
  end
  if connected
     % -----------------------------------------------------------------------
     % ---- read out system information 
     % -----------------------------------------------------------------------
    disp('Get System Info Data')
    calllib(libraryname, 'iV_GetSystemInfo', pSystemInfoData)
    get(pSystemInfoData, 'Value')
    % ------------------------------------------------------------------------
    % ---- start the calibration and validation process
    % ------------------------------------------------------------------------
    %NOTE: This process will be repeated until either user is satisfied with 
    %calibration processes or deviation parameters are below 0.5 degrees
    gazeCalibDataFD = createGazeCalibrationLog('C:\Users\student\Desktop\Infrastructure\Stimuli');
    while(~calibrated)
      disp('Calibrate iView X (eyetracking-server)')
      calllib(libraryname, 'iV_SetupCalibration', pCalibrationData)
      disp('Please press spacebar when you are ready for calibration to begin.')
      calllib(libraryname, 'iV_Calibrate')
      disp('Validate Calibration')
      calllib(libraryname, 'iV_Validate')
      disp('Show Accuracy')
      calllib(libraryname, 'iV_GetAccuracy', pAccuracyData, int32(0))
      accuracyStruct = libstruct('AccuracyStruct', pAccuracyData);
      get(pAccuracyData, 'Value')
      %Check calibration parameters if any are below the optimal 0.5 degrees 
      %let the user know.
      if((accuracyStruct.deviationLX > 0.5) ||(accuracyStruct.deviationLY > 0.5) ||...
         (accuracyStruct.deviationRX > 0.5) ||...
         (accuracyStruct.deviationRY > 0.5))
        calibrated = questdlg(['WARNING: The program has detected that',... 
                               ' one of the calibration deviations',...
                               ' exceeds 0.5. It is strongly recommended',... 
                               ' that you re-do the calibration procedure.',...
                               ' However, you may continue at your own risk.',...
                               'Would you like to re-do the calibration?'],... 
                               'Calibration Warning','Calibrate Again',... 
                               'Continue Experiment', ... 
                               'Calibrate Again');
        %If user decides to calibrated again, we change the valud of 
        %calibrated back to 0 so we do not break out of the calibration 
        %loop
        if(strcmpi('Calibrate Again', calibrated))
          calibrated = 0;
        %If subject choose to not calibrate, continue experiment, and 
        %save calibration data
        elseif(strcmpi('Continue Experiment', calibrated))
          calibrated = 1; 
          %The 1 in the function call is simply the subject number.
          writeCalibrationData(1, gazeCalibDataFD,... 
                                 accuracyStruct);
          disp('Successfully recorded calibration data');
        %control will never reach here
        else
          return
        end
      %Parameters were less than 0.5, so we can just proceed
      else
        calibrated = 1;
        disp('Calibration passed successfully.')
        writeCalibrationData(1, gazeCalibDataFD,... 
                                 accuracyStruct);
        disp('Successfully recorded calibration data');
      end
    end


  end
  %Once everything is setup, prepare the data collection struct.
  gazeDataStructure = struct('timeStamp', [], 'lGazeX', [], 'lGazeY', [], ...
                               'eyeDiameter', [], 'dist', []);
  %External screen number.
  screenNumber = 2; 
  try
    stimuliPath = 'C:\Users\student\Desktop\Infrastructure\Stimuli'
    gazeDataFD = createGazeDataLog(stimuliPath);
    windowPointer = Screen('OpenWindow', screenNumber);
    imageMatrix = imread(pathToFixationCross);
    texture = Screen('MakeTexture', windowPointer, imageMatrix);
    Screen('DrawTexture', windowPointer, texture);
    %Commence recording process
    calllib(libraryname, 'iV_ClearRecordingBuffer');
    calllib(libraryname, 'iV_StartRecording');
    %display stimulus
    Screen('Flip', windowPointer);
    %Commence stopwatch
    tic;
    %get value
    currentTime = toc;
    %Show stimulus for 2 seconds, so in theory, we should obtain 120
    %samples, since we have a frequency of 60 Hz.
    while(currentTime <= 2)
      returnValue = calllib(libraryname, 'iV_GetSample', pSampleData);
      if(returnValue ~=2 && returnValue == 1)
          sample = libstruct('SampleStruct', pSampleData);
          gazeDataStructure.timeStamp = [gazeDataStructure.timeStamp; ...
                                         sample.timestamp];
          gazeDataStructure.lGazeX = [gazeDataStructure.lGazeX; ...
                                      sample.leftEye.gazeX];
          gazeDataStructure.lGazeY = [gazeDataStructure.lGazeY; ...
                                      sample.leftEye.gazeY];
          gazeDataStructure.eyeDiameter = [gazeDataStructure.eyeDiameter;...
                                           sample.leftEye.diam];
          gazeDataStructure.dist = [gazeDataStructure.dist; sample.leftEye.eyePositionZ];
      end
      currentTime = toc;
    end
    calllib(libraryname, 'iV_StopRecording');
    % save recorded data
    user = 'User';
    description = 'Eye Tracker Validity';
    ovr = int32(1);
    filename = ['C:\Users\student\Desktop\Infrastructure\Results\IDF_Files\' 'testData' '.idf'];
    fprintf(gazeDataFD, '%f,%f,%f,%f\r\n', [gazeDataStructure.timeStamp,...
                                            gazeDataStructure.lGazeX,...
                                            gazeDataStructure.lGazeY,...
                                            gazeDataStructure.eyeDiameter,...
                                            ]');
    fclose(gazeDataFD);
    calibrationData = 2;
    exportToMasterFile(gazeCalibDataFD, stimuliPath, calibrationData);
    averageSubjectDistance = mean(gazeDataStructure.dist)/10; %convert from mm to cm
    fprintf('Average subject distance: %f cm\n', averageSubjectDistance);
    %Convert to radians
    deviationYRadians = (accuracyStruct.deviationLY * pi)/180;
    ret = calllib(libraryname, 'iV_SaveData', filename, description, user, ovr);
    disp(ret);
    [width, height] = Screen('WindowSize', screenNumber);
    %The function that we had intended to use to collect the physical height of
    %the display did not return the correct values. As such, we specified our 
    %value instead of using a function call.
    physHeight = 29.61 %CENTIMETERS
    yPixelFactor = height/physHeight; %Gives us how many pixels per cm
    %Collect mid points in pixels
    heightMP = height/2;
    widthMP = width/2;
    yErrorVal = averageSubjectDistance*tan(deviationYRadians)*yPixelFactor;
    yPosTolerance = heightMP + yErrorVal;
    yNegTolerance = heightMP - yErrorVal;
    %GRAPH DATA
    figure;
    whitebg('black');
    plot(gazeDataStructure.timeStamp, gazeDataStructure.lGazeX,'magenta',... 
         gazeDataStructure.timeStamp, widthMP*ones(length(gazeDataStructure.timeStamp)),'cyan');
    title('X-Position vs. Time');
    xlabel('Time(seconds)');
    ylabel('X-Position');
    legend('Left Eye Gaze X', 'width/2');
    figure;
    whitebg('black');
    plot(gazeDataStructure.timeStamp, gazeDataStructure.lGazeY,'green',...
         gazeDataStructure.timeStamp, heightMP*ones(length(gazeDataStructure.timeStamp)), 'red',...
         gazeDataStructure.timeStamp, yPosTolerance*ones(length(gazeDataStructure.timeStamp)), 'yellow',...
         gazeDataStructure.timeStamp, yNegTolerance*ones(length(gazeDataStructure.timeStamp)), 'magenta');
    title('Y-Position vs. Time');
    xlabel('Time(seconds)');
    ylabel('Y-Position');
    legend('Left Eye Gaze Y', 'height/2', 'yPosTolerance', 'yNegTolerance');
    fprintf('Upper bound: %f\nLowerBound: %f\n', yPosTolerance, yNegTolerance);
    fclose('all');
    Screen('Close', texture);
    Screen('CloseAll');
  catch
      fprintf('Error\n');
      sca;
      fclose('all');
      psychrethrow(psychlasterror);
  end
  



